package org.dbxml;

/*
 * dbXML License, Version 1.0
 *
 *
 * Copyright (c) 1999-2001 The dbXML Group, L.L.C.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by
 *        The dbXML Group (http://www.dbxml.com/)."
 *    Alternately, this acknowledgment may appear in the software
 *    itself, if and wherever such third-party acknowledgments normally
 *    appear.
 *
 * 4. The names "dbXML" and "The dbXML Group" must not be used to
 *    endorse or promote products derived from this software without
 *    prior written permission. For written permission, please contact
 *    info@dbxml.com.
 *
 * 5. Products derived from this software may not be called "dbXML",
 *    nor may "dbXML" appear in their name, without prior written
 *    permission of The dbXML Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE DBXML GROUP OR ITS CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
 * BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: Stopwatch.java,v 1.8 2001/07/10 00:52:17 tom Exp $
 */

/**
 * Stopwatch is a debugging tool for manually profiling code execution.
 * This class will probably be removed for a production release.
 */

public final class Stopwatch {
   String label = null;
   long total = 0;
   long begin = 0;
   long end = 0;

   public Stopwatch() {
   }

   public Stopwatch(boolean immediate) {
      if ( immediate )
         start();
   }
   
   public Stopwatch(String label) {
      this.label = label;
   }

   public Stopwatch(String label, boolean immediate) {
      this.label = label;
      if ( immediate )
         start();
   }
   
   public void start() {
      begin = System.currentTimeMillis();
   }
   
   public long stop() {
      end = System.currentTimeMillis();
      total += (end-begin);
      return total;
   }
   
   public void cancel() {
      begin = 0;
      end = 0;
   }
   
   public void reset() {
      total = 0;
      begin = 0;
      end = 0;
   }
   
   public long elapsed() {
      if ( end != 0 )
         return (end-begin);
      else
         return System.currentTimeMillis()-begin;
   }
   
   public long total() {
      if ( end != 0 )
         return total;
      else
         return (System.currentTimeMillis()-begin) + total;
   }
   
   public String toString() {
      long t = total();

      StringBuffer sb = new StringBuffer();
      if ( label != null )
         sb.append(label+": ");
      
      long hour = t / 3600000;
      if ( hour > 0 ) {
         sb.append(hour+"h ");
         t = t % 3600000;
      }
      
      long min = t / 60000;
      if ( min > 0 ) {
         sb.append(min+"m ");
         t = t % 60000;
      }
      
      long sec = t / 1000;
      if ( sec > 0 ) {
         sb.append(sec+"s ");
         t = t % 1000;
      }
      
      if ( t > 0 )
         sb.append(t+"ms");
      
      return sb.toString();
   }
}

