package org.dbxml.client.xmldb.services;

/*
 * dbXML License, Version 1.0
 *
 *
 * Copyright (c) 1999-2001 The dbXML Group, L.L.C.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by
 *        The dbXML Group (http://www.dbxml.com/)."
 *    Alternately, this acknowledgment may appear in the software
 *    itself, if and wherever such third-party acknowledgments normally
 *    appear.
 *
 * 4. The names "dbXML" and "The dbXML Group" must not be used to
 *    endorse or promote products derived from this software without
 *    prior written permission. For written permission, please contact
 *    info@dbxml.com.
 *
 * 5. Products derived from this software may not be called "dbXML",
 *    nor may "dbXML" appear in their name, without prior written
 *    permission of The dbXML Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE DBXML GROUP OR ITS CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
 * BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: XMLObjectService.java,v 1.7 2001/06/07 02:08:15 tom Exp $
 */

import org.dbxml.xml.*;
import org.dbxml.client.xmldb.*;
import org.dbxml.core.FaultCodes;
import org.dbxml.core.corba.db.*;
import org.dbxml.core.corba.EncodedBufferConverter;

import org.w3c.dom.Document;

import org.xmldb.api.base.*;
import org.xmldb.api.base.Collection;

/**
 * XMLObjectService enables invocation of XMLObjects within the dbXML server.
 *
 * The implementation of this functionality is currently quite simplistic.
 */
public class XMLObjectService extends CommonConfigurable
      implements Service {
   protected Collection collection = null;
   
   /**
    * Creates a new XMLObjectService to enable invocation of XMLObjects on the
    * server.
    */
   public XMLObjectService() {
   }

   public String getName() {
      return "XMLObjectService";
   }

   public String getVersion() {
      return "1.0";
   }

   /**
    * Sets the collection associated with this service.
    */
   public void setCollection(org.xmldb.api.base.Collection col) {
      this.collection = col;
   }

   /**
    * Invokes an XMLObject using the provided URI.
    *
    * @param the uri including parameters for the XMLObject to invoke.
    * @return a Resource encapsulating the result of the invocation. The
    * resource type is determined by the current resourceType setting of the
    * parent collection.
    * @exception XMLDBException Thrown if any error occurs.
    */
   public Resource invokeXMLObject(String uri) throws XMLDBException {
      try {
         // Create an empty Resource to hold the result of the invocation
         Resource result = collection.createResource(null, "XMLResource");
         
         // Invoke the object
         CollectionImpl col = (CollectionImpl) collection;
         EncodedBuffer buf = col.getServerObject().invokeXMLObject(uri);
         
         // TODO: we should make sure that no argument exceptions are thrown
         // TODO: properly handle result types.
         result.setContent(new String(buf.buf));
         return result;
      }
      catch (Exception e) {
         throw FaultCodes.createXMLDBException(e);
      }
   }

}

