package org.dbxml.client.xmldb.services;

/*
 * dbXML License, Version 1.0
 *
 *
 * Copyright (c) 1999-2001 The dbXML Group, L.L.C.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by
 *        The dbXML Group (http://www.dbxml.com/)."
 *    Alternately, this acknowledgment may appear in the software
 *    itself, if and wherever such third-party acknowledgments normally
 *    appear.
 *
 * 4. The names "dbXML" and "The dbXML Group" must not be used to
 *    endorse or promote products derived from this software without
 *    prior written permission. For written permission, please contact
 *    info@dbxml.com.
 *
 * 5. Products derived from this software may not be called "dbXML",
 *    nor may "dbXML" appear in their name, without prior written
 *    permission of The dbXML Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE DBXML GROUP OR ITS CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
 * BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: XPathQueryServiceImpl.java,v 1.18 2001/08/08 04:57:28 kstaken Exp $
 */

import java.util.*;

import org.dbxml.core.FaultCodes;
import org.dbxml.core.corba.db.*;
import org.dbxml.core.corba.*;
import org.dbxml.core.corba.db.Collection;
import org.dbxml.client.xmldb.*;

import org.dbxml.client.xmldb.resources.*;

import org.dbxml.xml.*;
import org.dbxml.xml.dom.*;

import org.w3c.dom.*;
import org.xmldb.api.base.*;
import org.xmldb.api.modules.*;

public class XPathQueryServiceImpl extends CommonConfigurable
   implements org.xmldb.api.modules.XPathQueryService
{
   private SymbolDeserializer syms;
   private NamespaceMap nsMap = new NamespaceMap();

   protected org.xmldb.api.base.Collection collection = null;

   public XPathQueryServiceImpl() {
   }

   public String getName() throws XMLDBException {
      return "XPathQueryService";
   }

   public String getVersion() throws XMLDBException {
      return "1.0";
   }

   public void setCollection(org.xmldb.api.base.Collection col)
         throws XMLDBException {
      this.collection = col;
   }

   public void setSymbolDeserializer(SymbolDeserializer syms) {
      this.syms = syms;
   }

   public void setDefaultNamespace(String uri) throws XMLDBException {
      if ( uri != null )
         nsMap.setDefaultNamespace(uri);
      else
         nsMap.removeDefaultNamespace();
   }
   
   public void removeDefaultNamespace() {
      nsMap.removeDefaultNamespace();
   }
   
   public void setNamespace(String prefix, String uri) throws XMLDBException {
      if ( prefix.equals("") || ( prefix == null ) ) {
         setDefaultNamespace(uri);
      }
      else {
         if ( uri != null ) {
            nsMap.setNamespace(prefix, uri);
         }
      }
   }
   
   public void removeNamespace(String prefix) {
      if ( prefix.equals("") || ( prefix == null ) ) {
         removeDefaultNamespace();
      }
      
      nsMap.removeNamespace(prefix);
   }
   
   public String getDefaultNamespace() {
      return nsMap.getDefaultNamespaceURI();
   }
   
   public String getNamespace(String prefix) {
      if ( prefix.equals("") || ( prefix == null ) ) {
         return nsMap.getDefaultNamespaceURI();
      }
   
      return nsMap.getNamespaceURI(prefix);
   }
   
   public void clearNamespaces() {
      nsMap.clear();
   }

   private NamedVal[] getNamespaces() {
      NamedVal[] nv = new NamedVal[nsMap.size()];
      Iterator iter = nsMap.keySet().iterator();
      int i = 0;
      while ( iter.hasNext() ) {
         String prefix = (String)iter.next();
         String uri = nsMap.getNamespaceURI(prefix);
         nv[i++] = new NamedVal(prefix, uri);
      }
      return nv;
   }
   
   /**
    * Executes an XPath query on the server. The results are returned as an
    * ResourceIterator
    *
    * @param query the XPath query string to execute.
    * @exception XMLDBException
    */
   public org.xmldb.api.base.ResourceSet query (String query)
         throws XMLDBException {
      // Get the dbXML collection object from the XML:DB collection wrapper.
      Collection col = ((CollectionImpl) collection).getServerObject();

      ResourceSet result = null;
      try {
         EncodedBuffer buffer = col.queryCollection("XPath", query, 
            getNamespaces(), syms.getLastModified());

         if ( buffer.stamp != -1 ) {
            SymbolTable s = syms.getSymbols(buffer);
            result = new ResourceSetImpl(collection, s, buffer.buf);
         }
         else
            result = new ResourceSetImpl(collection,
               EncodedBufferConverter.convertToDocument(buffer));
      }
      catch (Exception e) {
         result = new ResourceSetImpl(collection, null);
      }

      return result;
   }
   
   public org.xmldb.api.base.ResourceSet queryResource (String id, String query) 
         throws XMLDBException {
      // Get the dbXML collection object from the XML:DB collection wrapper.
      Collection col = ((CollectionImpl) collection).getServerObject();

      ResourceSet result = null;
      try {
         EncodedBuffer buffer = col.queryDocument("XPath", query, 
            getNamespaces(), id, syms.getLastModified());

         if ( buffer.stamp != -1 ) {
            SymbolTable s = syms.getSymbols(buffer);
            result = new ResourceSetImpl(collection, s, buffer.buf);
         }
         else
            result = new ResourceSetImpl(collection,
               EncodedBufferConverter.convertToDocument(buffer));
      }
      catch (Exception e) {
         result = new ResourceSetImpl(collection, null);
      }

      return result;
   }
   
   public XMLResource queryResult(String query) throws XMLDBException {
      Collection col = ((CollectionImpl) collection).getServerObject();

      XMLResource result = null;
      try {
         EncodedBuffer buffer = col.queryCollection("XPath", query, getNamespaces(), syms.getLastModified());

         if ( buffer.stamp != -1 ) {
            SymbolTable s = syms.getSymbols(buffer);
            result = new XMLResourceImpl("", collection, s, buffer.buf);
         }
         else
            result = new XMLResourceImpl("", collection, new String(buffer.buf));
      }
      catch (Exception e) {
         result = null;
      }

      return result;
   }
}


