package org.dbxml.tools.command;

/*
 * dbXML License, Version 1.0
 *
 *
 * Copyright (c) 1999-2001 The dbXML Group, L.L.C.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by
 *        The dbXML Group (http://www.dbxml.com/)."
 *    Alternately, this acknowledgment may appear in the software
 *    itself, if and wherever such third-party acknowledgments normally
 *    appear.
 *
 * 4. The names "dbXML" and "The dbXML Group" must not be used to
 *    endorse or promote products derived from this software without
 *    prior written permission. For written permission, please contact
 *    info@dbxml.com.
 *
 * 5. Products derived from this software may not be called "dbXML",
 *    nor may "dbXML" appear in their name, without prior written
 *    permission of The dbXML Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE DBXML GROUP OR ITS CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
 * BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: DeleteDocument.java,v 1.36 2001/06/28 23:29:05 kalafus Exp $
 */

import java.util.Hashtable;
import java.io.*;

import org.dbxml.client.xmldb.*;
import org.dbxml.client.xmldb.services.*;

import org.xmldb.api.base.*;
import org.xmldb.api.DatabaseManager;

import org.w3c.dom.*;

/**
 * DeleteDocument.java is designed to let the user delete or drop
 * a Document, referred by the Key, from a specific Collection or Nested
 * Collection.
 */

public class DeleteDocument extends Command {

    public boolean execute(Hashtable table ) throws Exception {
        
        Collection col = null;
        
        try {

            BufferedReader in = new BufferedReader(new InputStreamReader(System.in));

            if (table.get("collection") != null) {

                // Get a Collection reference to the collection to be deleted
                String colstring = normalizeCollectionURI( (String)table.get("collection") );
                
                col = DatabaseManager.getCollection( colstring );
                if ( col == null ) {
                  System.out.println("ERROR : Collection not found!");
                  return false;
                }
            
                if (table.get("nameOf") != null) {
                
                    // If the user didn't chools to force deletion, ASK
                    if (!table.get("force").equals("yes")) {
                        System.out.println("Continue deletion process? (y/n)");
            
                        try {
                            final String ans = in.readLine();
            
                            if (!ans.equalsIgnoreCase("y")) {
                                System.out.println("Operation aborted.");
                                return true;
                            }
            
                        } catch (IOException ie) {
                            System.out.println(ie);
                            return false;
                        }

                    }
                    
                     Resource colresource = col.getResource((String)table.get("nameOf"));
                        
                     col.removeResource(colresource);
                     System.out.println("DELETED: " + table.get("collection") + "/" + table.get("nameOf"));
                }
                
                else {
                    // User did not supply document name
                    System.out.println("ERROR : Document Key and switch required");
                }
                
            }
            else
                // User did not supply collection name
                System.out.println("ERROR : Collection and switch required");

        } catch (Exception e) {
            System.out.println("ERROR : " + e.getMessage() );
            if (table.get("verbose").equals("true")) {
              e.printStackTrace(System.out);
            }
            return false;
            
        } finally {
            if ( col != null ) {
                col.close();
            }
        }

      return true;
   }
}
