package org.dbxml.xml;

/*
 * dbXML License, Version 1.0
 *
 *
 * Copyright (c) 1999-2001 The dbXML Group, L.L.C.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by
 *        The dbXML Group (http://www.dbxml.com/)."
 *    Alternately, this acknowledgment may appear in the software
 *    itself, if and wherever such third-party acknowledgments normally
 *    appear.
 *
 * 4. The names "dbXML" and "The dbXML Group" must not be used to
 *    endorse or promote products derived from this software without
 *    prior written permission. For written permission, please contact
 *    info@dbxml.com.
 *
 * 5. Products derived from this software may not be called "dbXML",
 *    nor may "dbXML" appear in their name, without prior written
 *    permission of The dbXML Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE DBXML GROUP OR ITS CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
 * BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: XMLCompressedOutput.java,v 1.8 2001/08/05 07:23:10 tom Exp $
 */

import org.dbxml.xml.dom.*;
import org.dbxml.core.*;
import org.dbxml.server.*;

import org.w3c.dom.*;
import java.io.*;

/**
 * XMLCompressedInput is an OutputStream extension that provides functions for
 * writing types to a dbXML Compressed XML Stream.
 */

public class XMLCompressedOutput extends DataOutputStream {
   protected SymbolTable st = null;

   public XMLCompressedOutput(OutputStream os, SymbolTable st) {
      super(os);
      this.st = st;
   }

   /**
    * getSizeType returns the optimal size type for the specifed
    * length value.
    *
    * @see org.dbxml.xml.Signatures
    *
    * @param length The length to evaluate
    * @return The optimal size
    */
   protected final byte getSizeType(long length) {
      if ( length > Short.MAX_VALUE )
         return Signatures.IntContent;
      else if ( length > Byte.MAX_VALUE )
         return Signatures.ShortContent;
      else if ( length > 0 )
         return Signatures.ByteContent;
      else
         return Signatures.NoContent;
   }

   /**
    * getSizeSize returns the size (in bytes) of the specified
    * size type.
    *
    * @see org.dbxml.xml.Signatures
    *
    * @param sizeType The size type
    * @return The size in bytes
    */
   protected final int getSizeSize(int sizeType) {
      switch ( sizeType ) {
         case Signatures.IntContent:   return 4;
         case Signatures.ShortContent: return 2;
         case Signatures.ByteContent:  return 1;
         default:                      return 0;
      }
   }

   /**
    * writeSize writes the size to the output stream based on the
    * specified sizeType.
    *
    * @see org.dbxml.xml.Signatures
    *
    * @param sizeType The size type
    * @param size The size
    * @return The size in bytes written
    * @throws IOException If the write failed
    */
   protected final int writeSize(int sizeType, int size) throws IOException {
      switch ( sizeType ) {

         case Signatures.IntContent:
            writeInt(size);
            return 4;

         case Signatures.ShortContent:
            writeShort((short)size);
            return 2;

         case Signatures.ByteContent:
            writeByte((byte)size);
            return 1;

         default:
            return 0;
      }
   }
}


