package org.dbxml.xml.dom;

/*
 * dbXML License, Version 1.0
 *
 *
 * Copyright (c) 1999-2001 The dbXML Group, L.L.C.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by
 *        The dbXML Group (http://www.dbxml.com/)."
 *    Alternately, this acknowledgment may appear in the software
 *    itself, if and wherever such third-party acknowledgments normally
 *    appear.
 *
 * 4. The names "dbXML" and "The dbXML Group" must not be used to
 *    endorse or promote products derived from this software without
 *    prior written permission. For written permission, please contact
 *    info@dbxml.com.
 *
 * 5. Products derived from this software may not be called "dbXML",
 *    nor may "dbXML" appear in their name, without prior written
 *    permission of The dbXML Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE DBXML GROUP OR ITS CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
 * BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: CompressedDocument.java,v 1.9 2001/08/05 07:23:15 tom Exp $
 */

import org.w3c.dom.*;
import org.dbxml.xml.*;

/**
 * CompressedDocument exposes the Document's SymbolTable.
 */

public interface CompressedDocument extends CompressedNode, Document {
   /**
    * getSymbols returns the Symbol table for this Document.  A new
    * Document may not have a Symbol table as Symbol tables only come
    * into play when a Document has been compressed or decompressed.
    *
    * @return The Symbol table
    */
   SymbolTable getSymbols();

   /**
    * setSymbols sets the Symbol table for this Document.  Setting
    * the Symbol table for a Document that already has a Symbol table
    * could be VERY BAD.  Generally, an implementation should throw
    * a DOMException if an attempt is made to reassign a Symbol table.
    *
    * @param symbols The Symbol table
    * @throws DOMException if an exception occurs
    */
   void setSymbols(SymbolTable symbols) throws DOMException;

   /**
    * getNodeAtPos returns the Node that exists at the specified
    * byte data  position in the compressed stream.  This method
    * should only be used on an unmodified Document as the
    * underlying compressed stream of a document will become
    * invalid as the Document is modified.
    *
    * @param pos The compressed stream position
    * @return The Node (or null if not found)
    */
   Node getNodeAtPos(int pos);
}
