package org.dbxml.xml.dom;

/*
 * dbXML License, Version 1.0
 *
 *
 * Copyright (c) 1999-2001 The dbXML Group, L.L.C.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by
 *        The dbXML Group (http://www.dbxml.com/)."
 *    Alternately, this acknowledgment may appear in the software
 *    itself, if and wherever such third-party acknowledgments normally
 *    appear.
 *
 * 4. The names "dbXML" and "The dbXML Group" must not be used to
 *    endorse or promote products derived from this software without
 *    prior written permission. For written permission, please contact
 *    info@dbxml.com.
 *
 * 5. Products derived from this software may not be called "dbXML",
 *    nor may "dbXML" appear in their name, without prior written
 *    permission of The dbXML Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE DBXML GROUP OR ITS CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
 * BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: EntityImpl.java,v 1.9 2001/09/22 23:41:30 tom Exp $
 */

import org.w3c.dom.*;

/**
 * EntityImpl
 */

public final class EntityImpl extends NodeImpl implements Entity {
   private String encoding = null;
   private String version = null;
   
   public EntityImpl() {
   }
   
   public EntityImpl(NodeImpl parentNode, byte[] data, int pos, int len) {
      super(parentNode, data, pos, len);
   }

   public EntityImpl(NodeImpl parentNode, boolean dirty) {
      super(parentNode, dirty);
   }

   public short getNodeType() {
      return Node.ENTITY_NODE;
   }

   /**
    * The public identifier associated with the entity, if specified. If the
    * public identifier was not specified, this is <code>null</code>.
    */
   public String getPublicId() {
      return null;
   }

   /**
    * The system identifier associated with the entity, if specified. If the
    * system identifier was not specified, this is <code>null</code>.
    */
   public String getSystemId() {
      return null;
   }

   /**
    * For unparsed entities, the name of the notation for the entity. For
    * parsed entities, this is <code>null</code>.
    */
   public String getNotationName() {
      return null;
   }
   
   // DOM Level 3 Stuff
   
   public String getEncoding() {
      checkLoaded();
      return encoding;
   }
   
   public void setEncoding(String encoding) {
      checkReadOnly();
      checkLoaded();
      this.encoding = encoding;
   }
   
   public String getVersion() {
      checkLoaded();
      return version;
   }
   
   public void setVersion(String version) {
      checkReadOnly();
      checkLoaded();
      this.version = version;
   }
}



