package org.dbxml.xml.sax;

/*
 * dbXML License, Version 1.0
 *
 *
 * Copyright (c) 1999-2001 The dbXML Group, L.L.C.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by
 *        The dbXML Group (http://www.dbxml.com/)."
 *    Alternately, this acknowledgment may appear in the software
 *    itself, if and wherever such third-party acknowledgments normally
 *    appear.
 *
 * 4. The names "dbXML" and "The dbXML Group" must not be used to
 *    endorse or promote products derived from this software without
 *    prior written permission. For written permission, please contact
 *    info@dbxml.com.
 *
 * 5. Products derived from this software may not be called "dbXML",
 *    nor may "dbXML" appear in their name, without prior written
 *    permission of The dbXML Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE DBXML GROUP OR ITS CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
 * BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: CompressionHandler.java,v 1.6 2001/08/05 07:23:26 tom Exp $
 */

import org.dbxml.xml.*;
import org.dbxml.xml.dom.*;
import org.dbxml.server.util.*;

import org.w3c.dom.*;
import org.xml.sax.*;
import java.io.*;
import java.util.*;

/**
 * CompressionHandler is a SAX callback interface for reporting
 * positional information about dbXML compressed Documents.  All
 * of these methods will be called immediate before any SAX start
 * methods.
 */

public interface CompressionHandler {
   static final String HANDLER = "CompressionHandler/1.0";

   /**
    * symbols reports the Symbol table that is being used for the
    * decompression.  This method should only be called when the
    * Symbol table changes in the stream.  For example, before and
    * after an imported Node is enountered that uses a diffferent
    * Symbol table.
    *
    * @param symbols The Symbol Table
    */
   void symbols(SymbolTable symbols);

   /**
    * dataBytes sets the byte array for the definitions to follow.
    * This method should only be called when the byte array object
    * changes in the stream.  For example, before and after an
    * imported Node is encountered that uses a different byte array.
    *
    * @param data The Data
    */
   void dataBytes(byte[] data);

   /**
    * dataLocation reports positional information for the Node to
    * follow.
    *
    * @param pos The offset into the data for this Node
    * @param len The length of this Node's definition
    */
   void dataLocation(int pos, int len);

   /**
    * symbolID reports the SymbolID for the Node to follow.  Because
    * Symbol IDs are specific to a Node type (Element, Entity, and
    * Attribute) this value needs to be stored until the type of Node
    * is reported.
    *
    * @param symbol The Symbol ID
    */
   void symbolID(short symbolID);
}
